@@ -17,18 +17,18 @@ from compat import unittest
 from distlib import __version__
 from distlib.compat import StringIO
 from distlib.metadata import (LegacyMetadata, Metadata, METADATA_FILENAME,
-                              PKG_INFO_PREFERRED_VERSION,
-                              MetadataConflictError, MetadataMissingError,
-                              MetadataUnrecognizedVersionError,
+                              LEGACY_METADATA_FILENAME, PKG_INFO_PREFERRED_VERSION,
+                              MetadataMissingError, MetadataUnrecognizedVersionError,
                               MetadataInvalidError, _ATTR2FIELD)
 
-from support import (LoggingCatcher, TempdirManager)
+from support import LoggingCatcher, TempdirManager, DistlibTestCase, in_github_workflow
 
 
 HERE = os.path.abspath(os.path.dirname(__file__))
+IN_GITHUB_WORKFLOW = in_github_workflow()
 
 class LegacyMetadataTestCase(LoggingCatcher, TempdirManager,
-                             unittest.TestCase):
+                             DistlibTestCase):
 
     maxDiff = None
     restore_environ = ['HOME']
@@ -281,8 +281,11 @@ class LegacyMetadataTestCase(LoggingCatcher, TempdirManager,
         metadata.set_metadata_version()
         self.assertEqual(metadata['Metadata-Version'], '1.2')
         metadata.set('Obsoletes', 'ok')
-        self.assertRaises(MetadataConflictError,
-                          metadata.set_metadata_version)
+        # See issue #140. Relaxed checking on Obsoletes
+        # self.assertRaises(MetadataConflictError,
+                          # metadata.set_metadata_version)
+        metadata.set_metadata_version()
+        self.assertEqual(metadata['Metadata-Version'], '2.0')
 
         del metadata['Obsoletes']
         del metadata['Obsoletes-Dist']
@@ -335,6 +338,7 @@ class LegacyMetadataTestCase(LoggingCatcher, TempdirManager,
         self.assertEqual(1, len(logs))
         self.assertIn('not a valid version', logs[0])
 
+    @unittest.skipIf(IN_GITHUB_WORKFLOW, 'This test is end-of-line dependent')
     def test_description(self):
         content = self.get_file_contents('PKG-INFO')
         metadata = LegacyMetadata()
@@ -474,7 +478,7 @@ class LegacyMetadataTestCase(LoggingCatcher, TempdirManager,
         self.assertFalse(md.is_field('Frobozz'))
 
 class MetadataTestCase(LoggingCatcher, TempdirManager,
-                       unittest.TestCase):
+                       DistlibTestCase):
     def test_init(self):
         "Test initialisation"
         md = Metadata()
@@ -505,7 +509,7 @@ class MetadataTestCase(LoggingCatcher, TempdirManager,
 
         # Initialise from legacy metadata
         fn = os.path.join(HERE, 'fake_dists', 'choxie-2.0.0.9.dist-info',
-                          'METADATA')
+                          LEGACY_METADATA_FILENAME)
         md = Metadata(path=fn)
         md.validate()
         self.assertIsNotNone(md._legacy)
@@ -534,7 +538,7 @@ class MetadataTestCase(LoggingCatcher, TempdirManager,
         self.assertEqual(md.run_requires, [{ 'requires': ['foo (0.1.2)']}])
 
         fn = os.path.join(HERE, 'fake_dists', 'choxie-2.0.0.9.dist-info',
-                          'METADATA')
+                          LEGACY_METADATA_FILENAME)
         md = Metadata(path=fn)
         md.add_requirements(['foo (0.1.2)'])
         self.assertEqual(set(md.run_requires),
@@ -631,7 +635,7 @@ class MetadataTestCase(LoggingCatcher, TempdirManager,
         dfn = self.temp_filename()
         # Read legacy, write new
         sfn = os.path.join(HERE, 'fake_dists', 'choxie-2.0.0.9.dist-info',
-                           'METADATA')
+                           LEGACY_METADATA_FILENAME)
         md = Metadata(path=sfn)
         md.write(path=dfn)
         with codecs.open(dfn, 'r', 'utf-8') as f:
