@@ -19,11 +19,10 @@ import textwrap
 import time
 
 from compat import unittest
-
-from support import TempdirManager
+from support import TempdirManager, DistlibTestCase, in_github_workflow
 
 from distlib import DistlibException
-from distlib.compat import cache_from_source,  Container
+from distlib.compat import cache_from_source
 from distlib.util import (get_export_entry, ExportEntry, resolve,
                           get_cache_base, path_to_cache_dir, zip_dir,
                           parse_credentials, ensure_slash, split_filename,
@@ -35,7 +34,7 @@ from distlib.util import (get_export_entry, ExportEntry, resolve,
 
 
 HERE = os.path.dirname(os.path.abspath(__file__))
-
+IN_GITHUB_WORKFLOW = in_github_workflow()
 
 class TestContainer(object):
     def __init__(self, *args, **kwargs):
@@ -43,7 +42,7 @@ class TestContainer(object):
         self.kwargs = kwargs
 
 
-class UtilTestCase(unittest.TestCase):
+class UtilTestCase(DistlibTestCase):
     def check_entry(self, entry, name, prefix, suffix, flags):
         self.assertEqual(entry.name, name)
         self.assertEqual(entry.prefix, prefix)
@@ -61,6 +60,9 @@ class UtilTestCase(unittest.TestCase):
                          'foo', 'foo.bar', 'main', [])
         self.check_entry(get_export_entry('foo=foo.bar:main [a]'),
                          'foo', 'foo.bar', 'main', ['a'])
+        # See issue #127 - allow hyphens
+        self.check_entry(get_export_entry('foo=foo.bar:main [with-foo]'),
+                         'foo', 'foo.bar', 'main', ['with-foo'])
         self.check_entry(get_export_entry('foo=foo.bar:main [ a ]'),
                          'foo', 'foo.bar', 'main', ['a'])
         self.check_entry(get_export_entry('foo=foo.bar:main [a=b, c=d,e, f=g]'),
@@ -584,7 +586,11 @@ def _eta_range(min_eta, max_eta, prefix='ETA '):
     msg = prefix + ': 00:00:%02d'
     return tuple([msg % v for v in range(min_eta, max_eta + 1)])
 
-class ProgressTestCase(unittest.TestCase):
+class ProgressTestCase(DistlibTestCase):
+    # Of late, the speed tests keep failing on AppVeyor and Windows
+    @unittest.skipIf(IN_GITHUB_WORKFLOW or (os.name == 'nt' and
+                                            os.environ.get('APPVEYOR') == 'True'),
+                     'Test disabled on some environments due to performance')
     def test_basic(self):
 
         # These ranges may need tweaking to cater for especially slow
@@ -621,6 +627,10 @@ class ProgressTestCase(unittest.TestCase):
         self.assertIn(bar.ETA, e, p)
         self.assertIn(bar.speed, s)
 
+    # Of late, the speed tests keep failing on AppVeyor and Windows
+    @unittest.skipIf(IN_GITHUB_WORKFLOW or (os.name == 'nt' and
+                                            os.environ.get('APPVEYOR') == 'True'),
+                     'Test disabled on some environments due to performance')
     def test_unknown(self):
         if os.name == 'nt':
             speed = _speed_range(17, 20)
@@ -652,7 +662,7 @@ class ProgressTestCase(unittest.TestCase):
         self.assertEqual(bar.ETA, e)
         self.assertIn(bar.speed, s)
 
-class FileOpsTestCase(unittest.TestCase):
+class FileOpsTestCase(DistlibTestCase):
 
     def setUp(self):
         self.fileop = FileOperator()
@@ -745,7 +755,7 @@ class FileOpsTestCase(unittest.TestCase):
         self.assertFalse(self.fileop.record)
 
 
-class GlobTestCaseBase(TempdirManager, unittest.TestCase):
+class GlobTestCaseBase(TempdirManager, DistlibTestCase):
 
     def build_files_tree(self, files):
         tempdir = self.mkdtemp()
@@ -943,6 +953,12 @@ class GlobTestCase(GlobTestCaseBase):
                  'a == 1.2.*, != 1.2.1.*', None))
         r = parse_requirement('a @ http://domain.com/path#abc=def')
         validate(r, ('a', None, None, 'a', 'http://domain.com/path#abc=def'))
+        # See issue #148
+        r = parse_requirement('a >=3.6')
+        validate(r, ('a', [('>=', '3.6')], None, 'a >= 3.6', None))
+        r = parse_requirement('a >=3.6,')
+        validate(r, ('a', [('>=', '3.6')], None, 'a >= 3.6', None))
+
         if False: # TODO re-enable
             for e in ('*', ':*:', ':meta:', '-', '-abc'):
                 r = parse_requirement('a [%s]' % e)
